;;; vhdl-highlight.el --- Highlighting for VHDL code.
;;
;; Copyright (C) 1995 Ken Wood
;;
;; Author: Ken Wood <ken@eda.com.au>
;; Keywords: VHDL source code highlighting
;; Version: 2.60
;;
;;
;;
;; PURPOSE:
;;
;; Make VHDL source code buffers more readable by highlighting
;; constructs and keywords using a variety of colors & fonts.
;;
;; This package will work with either XEmacs 19.12 (or later), or GNU
;; Emacs 19.28 (or later). There are some known bugs in XEmacs 19.11
;; which will affect the operation of this package.
;;
;; See also the vhdl-mode, font-lock and lazy-lock packages.
;;
;;
;;
;; ARCHIVE:
;;
;; The latest version of this file is always available as:
;;
;;   ftp://ftp.eda.com.au/pub/emacs/vhdl-highlight.tar.gz
;;
;;
;;
;; INSTALLATION & CUSTOMISATION:
;;
;; 1. First you will need a VHDL editing mode. I recommend Rod
;;    Whitby's package, which can be obtained as:
;;
;;      ftp://ftp.eda.com.au/pub/emacs/vhdl-mode.tar.gz
;;    
;;    Install vhdl-mode as per the instructions that come with it.
;;
;;    Note that you need to bind the VHDL editing mode to appropriate
;;    file names. Although this is described in the vhdl-mode
;;    documentation, an example of how to do this is given below:
;;
;;    Add this code to your .emacs or another emacs-lisp file loaded
;;    at startup. You may wish to add additional file name
;;    expressions, or modify those used below, if your site uses a
;;    different naming convention for VHDL source files.
;;
;;      (setq auto-mode-alist
;;            (append '(
;;                      ;; VHDL source
;;                      ("\\.vhdl?$"            . vhdl-mode)
;;                      ("\\.hdl$"              . vhdl-mode)
;;                      ("\\.vhdl?_[0-9]+$"     . vhdl-mode)
;;                      ("\\.hdl_[0-9]+$"       . vhdl-mode)
;;                      )
;;                    auto-mode-alist
;;                    ))
;;
;;      (autoload 'vhdl-mode "vhdl-mode"
;;        "VHDL editing mode - autoloadable function" t)
;;
;; 2. Now load the setup file for VHDL highlighting. The actual
;;    package is not loaded until you edit a VHDL source file, making
;;    emacs startup faster. Byte-compile the files:
;;
;;      vhdl-highlight.el
;;      vhdl-highlight-setup.el
;;
;;    and put the resulting .elc files somewhere in your load-path.
;;    Now add the following to your .emacs:
;;
;;      (require 'vhdl-highlight-setup)
;;
;;
;; 3. OPTIONAL - For improved performance, I recommend also loading
;;    the lazy-lock package, which provides page-at-a-time
;;    highlighting on large source files. This package is included in
;;    the XEmacs 19.12 distribution, but you may need to enable it thus:
;;
;;      (add-hook 'font-lock-mode-hook 'turn-on-lazy-lock)
;;
;;    If your workstation is particularly slow, you may wish to reduce
;;    the file size threshold for incremental highlighting to, say 5k
;;    from the 10k default. Use this command in your .emacs:
;;
;;      (setq lazy-lock-minimum-size 5000)
;;
;;
;; 4. Choose fonts to use for highlighting.  See the documentation for
;;    font-lock-mode for more details, but here is what I've got in my
;;    .emacs, as an example:
;;
;;      ;; These variable settings have to come first, before the lines below.
;;      (setq font-lock-use-fonts t)
;;      (setq font-lock-use-colors nil)
;;      (setq font-lock-use-maximal-decoration t)
;;
;;      ;; Somewhere in your .emacs, font-lock will be loaded with a
;;      ;; statement like this:
;;      (require 'font-lock)
;;
;;      ;; After font-lock is loaded, you can set the colours and still use
;;      ;; italic & bold fonts as chosen.
;;      (set-face-foreground 'font-lock-comment-face       "blue3")
;;      (set-face-foreground 'font-lock-doc-string-face    "green4")
;;      (set-face-foreground 'font-lock-string-face        "green4")
;;      (set-face-foreground 'font-lock-function-name-face "red3")
;;      (set-face-foreground 'font-lock-variable-name-face "magenta3")
;;      (set-face-foreground 'font-lock-signal-name-face   "gold3")
;;      (set-face-foreground 'font-lock-type-face          "purple1")
;;
;;
;; 5. After using the package for a while, decide if you need to
;;    reduce the complexity of highlighting it performs in order to
;;    improve performance.
;;
;;    By default, a complete set of regexps is used for highlighting.
;;    This set highlights fairly quickly on any machine at least as
;;    fast as a Sparc 2.  You can choose a smaller set of highlighting
;;    expressions by adding one of the first two of the following
;;    lines to your .emacs:
;;
;;      (setq vhdl-font-lock-highlight-level 1)
;;
;;      (setq vhdl-font-lock-highlight-level 2)
;;
;;      (setq vhdl-font-lock-highlight-level 3)  ; this is the default
;;
;;    See the documentation for this variable, for more detail.
;;
;;
;;
;; KNOWN BUGS/LIMITATIONS:
;;
;; 1) VHDL-87 style END labels are not highlighted.
;; e.g.
;;
;;   entity <ent> is
;;   begin
;;   end <ent>;
;;
;; In this case, the final <ent> is not highlighted because I consider
;; it too computationally demanding to find & highlight this label.
;; However, the following VHDL-93 style termination is correctly
;; highlighted:
;;
;;   end entity <ent>;
;;
;;
;; 2) The package assumes statements appear one per line.  If you put
;; more than one statment on a line, only the first one will be
;; highlighted.  e.g. In the following, only the declaration of port A
;; is highlighted:
;;
;;   port( A : in std_logic;  Z : out std_logic);
;;
;; But, when this is split into two lines, both port declarations are
;; highlighted:
;;
;;   port( A, B : in std_logic;
;;         Z : out std_logic);
;;
;; It's too slow to check for multiple statements per line.
;;
;;
;; 3) Entity ports without a specified direction do not highlight.
;; e.g. the signal type will not highlight in this case:
;;
;;   port (in1 : std_logic;
;;
;; but it will highlight in both of these cases:
;;
;;   port (in1 : in std_logic;
;;
;;   port (signal in1 : std_logic;
;;
;;
;; 4) The package does not cope with multi-line statements in all
;; situations.  e.g. the return type is highlighted correctly here:
;;
;;   FUNCTION "+"(left, right : bit_vector) RETURN bit_vector IS
;;
;; But not here:
;;
;;   FUNCTION "+"(left, right : bit_vector) RETURN bit_vector
;;   IS
;;
;; This is a known limitation - it's too slow to check for multi-line
;; statements.
;;
;;
;; 5) Less-than-or-equal comparisons using <= may be confused with
;; signal assignements, and highlighted accordingly. This is a known
;; bug.
;;
;;




;;
;; CODE:
;;
;;
;;
;; Ensure some needed packages are already loaded before we execute
;; the font-lock setup code following.

(require 'font-lock)
(require 'vhdl-mode)


;; VHDL is case-insensitive
(put 'vhdl-mode 'font-lock-keywords-case-fold-search t)

(defconst vhdl-font-lock-keywords-1 nil
  "For consideration as a value of `vhdl-font-lock-keywords'. This
does highlighting on VHDL-files edited in vhdl-mode. Includes only
names of common constructs like entity, architecture, configuration
etc.")

(defconst vhdl-font-lock-keywords-2 nil
  "For consideration as a value of `vhdl-font-lock-keywords'. This
does highlighting on VHDL-files edited in vhdl-mode.  Includes signal,
variable and type names.")

(defconst vhdl-font-lock-keywords-3 nil
  "For consideration as a value of `vhdl-font-lock-keywords'.  This
does highlighting on VHDL-files edited in vhdl-mode.  Includes a
full list of reserved words, attributes and standard types.")

(defvar vhdl-font-lock-highlight-level 3
  "*Specify what level of complexity to use when highlighting VHDL code. The
higher the complexity the slower the highlighting. Possible values are:

nil - no VHDL-specific highlighting
1   - label and definition name highlighting only
2   - signal, variable, type, library etc highlighting also
3   - reserved word highlighting as well as the above

Level 2 highlighting should be quite responsive on all but the largest
source files. The highlighting complexity is updated only when files are
loaded.

If you are using page-at-a-time highlighting, such as in the lazy-lock
package, you should be able to use level 3 highlighting without any
performance problems.")


;; Some definitions for level-1 fontifying
(setq vhdl-font-lock-keywords-1
      (list


       ;; Highlight labels of common constructs
       (list
	(concat
	 ;; Look for a label symbol first & highlight
	 "^[ \t]*\\([a-zA-Z][a-zA-Z0-9_]*\\)"
	 ;; Look for colon flagging the construct
	 "[ \t]*:[ \t]*"
	 ;; Look for the contruct type
	 "\\(block\\|case\\|exit\\|for\\|if\\|loop\\|"
	 "next\\|null\\|process\\|with\\)"
	 ;; Look for symbol termination at the end of the construct name
	 "\\>[^_]")
	1 'font-lock-function-name-face)


       ;; Highlight labels of component instantiations
       (list
	(concat
	 ;; Look for a label symbol first & highlight
	 "^[ \t]*\\([a-zA-Z][a-zA-Z0-9_]*\\)"
	 ;; Look for colon flagging the construct
	 "[ \t]*:[ \t]*"
	 ;; Look for component name
	 "\\([a-zA-Z][a-zA-Z0-9_]*\\)"
	 ;; Look for port map or generic map to flag the construct
	 "[ \t]+\\(port\\|generic\\)[ \t]+map"
	 ;; Look for symbol termination at the end of the construct name
	 "\\>[^_]")
	1 'font-lock-variable-name-face)


       ;; Highlight component names in component instantiations
       (list
	(concat
	 ;; Look for a label symbol first & highlight
	 "^[ \t]*\\([a-zA-Z][a-zA-Z0-9_]*\\)"
	 ;; Look for colon flagging the construct
	 "[ \t]*:[ \t]*"
	 ;; Look for component name
	 "\\([a-zA-Z][a-zA-Z0-9_]*\\)"
	 ;; Look for port map or generic map to flag the construct
	 "[ \t]+\\(port\\|generic\\)[ \t]+map"
	 ;; Look for symbol termination at the end of the construct name
	 "\\>[^_]")
	2 'font-lock-function-name-face)


       ;; Highlight names of common constructs and end labels
       (list
	(concat
	 ;; Look for construct beginnings first
	 "^[ \t]*\\(architecture\\|component\\|configuration\\|entity\\|"
	 "function\\|procedure\\|package[ \t]+body\\|package\\|"
	 ;; Look for construct ends (VHDL 93 style) now
	 "end[ \t]+architecture\\|end[ \t]+block\\|end[ \t]+case\\|"
	 "end[ \t]+component\\|end[ \t]+configuration\\|end[ \t]+entity\\|"
	 "end[ \t]+for\\|end[ \t]+function\\|end[ \t]+generate\\|"
	 "end[ \t]+loop\\|end[ \t]+package[ \t]+body\\|end[ \t]+package\\|"
	 "end[ \t]+procedure\\|end[ \t]+process\\)"
	 ;; Finally, look for the construct label & highlight
	 "[ \t]+\\([a-zA-Z][a-zA-Z0-9_]*\\)")
	2 'font-lock-function-name-face)


       ;; Highlight OF labels
       (list
	(concat
	 ;; Look for construct beginning first
	 "^[ \t]*\\(configuration\\|architecture\\|alias\\|attribute\\)[ \t]+"
	 ;; Skip over the construct label
	 "\\([a-zA-Z][a-zA-Z0-9_]*\\)[ \t]*"
	 ;; Look for the "of" clause
	 "\\([ \t]+of\\|[ \t]+is\\|:\\)[ \t]+"
	 ;; Finally, find the "of" label & highlight
	 "\\([a-ZA-Z][a-zA-Z0-9_.()]*\\)")
	4 'font-lock-function-name-face)
       ))


;; Next some level-2 fontifying definitions.
(setq vhdl-font-lock-keywords-2
      (list


       ;; Fontify character literals as strings.
       (list
	"'\\(.\\)'"
	1 'font-lock-doc-string-face)


       ;; Fontify library clauses
       (list
	(concat
	 ;; Look for library command
	 "^[ \t]*library[ \t]+"
	 ;; Now look for the label & highlight
	 "\\([a-zA-Z][a-zA-Z0-9_]*\\)"
	 ;; Lastly, look for the terminator
	 "[ \t]*;")
	1 'font-lock-function-name-face)

       ;; Fontify use, and use entity clauses on their own lines
       (list
	(concat
	 ;; Look for "use" or "use entity" command
	 "^[ \t]*\\(use[ \t]+entity\\|use\\)[ \t]+"
	 ;; Now look for the label & highlight
	 "\\([a-zA-Z][a-zA-Z0-9_.() \t]+\\)+"
	 ;; Lastly, look for the terminator
	 "[ \t]*[;\n]")
	2 'font-lock-function-name-face)


       ;; Fontify "for [component] : use entity [entity]" statements
       (list
	(concat
	 ;; Look for the configuration preamble
	 "^[ \t]*for[ \t]+[a-zA-Z0-9_]+[ \t]*:[ \t]*[a-zA-Z0-9_]+[ \t]+"
	 ;; Look for use entity command
	 "use[ \t]+entity[ \t]+"
	 ;; Now look for the label & highlight
	 "\\([a-zA-Z][a-zA-Z0-9_.() \t]+\\)"
	 ;; Lastly, look for the terminator
	 "[ \t]*[;\n]")
	1 'font-lock-function-name-face)


       ;; Fontify aliases & attribute declarations
       (list
	(concat
	 ;; Look for the declaration type first
	 "^[ \t]*\\(alias\\|attribute\\)[ \t]+"
	 ;; Next, look for the declaration symbol & highlight
	 "\\([a-zA-Z][a-zA-Z0-9_]*\\)")
	2 'font-lock-variable-name-face)


       ;; Fontify FOR, CASE & WITH indices
       (list
	(concat
	 ;; Look for an optional label
	 "^[ \t]*\\([a-zA-Z][a-zA-Z0-9_]*[ \t]*:\\)?[ \t]*"
	 ;; Look for thy type declaration next
	 "\\(case\\|with\\|for\\)[ \t]+"
	 ;; Next, look for the declaration symbol & highlight
	 "\\([a-zA-Z][a-zA-Z0-9_.()]*\\)")
	3 'font-lock-variable-name-face)


       ;; Fontify WHEN arguments
       (list
	(concat
	 ;; Look for WHEN statement first
	 "^[ \t]*when[ \t]+"
	 ;; Look for the expression that we're going to highlight
	 "\\([^\n]+\\)"
	 ;; Check for the termination construct
	 "[ \t]*=>")
	1 'font-lock-doc-string-face)


       ;; Fontify variable names where they are used.
       (list
	(concat
	 ;; Look for the variable name near the beginning of line,
	 ;; and highlight
	 "^[ \t]*\\([a-zA-Z][a-zA-Z0-9_.() \t+-]*\\)"
	 ;; Check for the assignment statement
	 "[ \t]*:=")
	1 'font-lock-variable-name-face)


       ;; Highlight FOR loop ranges in configurations
       (list
	(concat
	 ;; Look for construct beginning first
	 "^[ \t]*for[ \t]+"
	 ;; Skip over the FOR index
	 "\\([a-zA-Z][a-zA-Z0-9_]*\\)"
	 ;; Look for the : separator
	 "[ \t]*:[ \t]*"
	 ;; Finally, find the range symbol & highlight
	 "\\([a-zA-Z][a-zA-Z0-9_]*\\)")
	2 'font-lock-function-name-face)


       ;; Fontify values in constant, signal, variable and file
       ;; declarations.
       (list
	(concat
	 ;; Skip over any function/procedure declarations
	 "^[a-zA-Z0-9_ \t]*[ \t(]+"
	 ;; Check for a constant, variable or file declaration.
	 "\\(variable\\|constant\\|signal\\|file\\)[ \t]+"
	 ;; Skip over constant/signal/variable/file names
	 "[a-zA-Z][a-zA-Z0-9_.,() \t+-]*"
	 ;; Skip over signal direction, if any
	 ":[ \t]*\\(inout \\|out \\|in \\|buffer \\)?[ \t]*"
	 ;; Skip over signal/whatever type and find assignment
	 "[a-zA-Z][a-zA-Z0-9_.'() \t+-]+:=[ \t]*"
	 ;; Find the assigned value & highlight
	 "\\([^:;\n]+\\)"
	 ;; Find the statement terminator
	 "[:;\n]")
	3 'font-lock-doc-string-face)


       ;; Fontify constant, variable and file names where they are
       ;; defined, regardless of whether this is part of a
       ;; function/procedure declaration or standalone.
       (list
	(concat
	 ;; Skip over any function/procedure declarations
	 "^[a-zA-Z0-9_ \t]*[ \t(]+"
	 ;; Check for a constant, variable or file declaration.
	 "\\(variable\\|constant\\|file\\)[ \t]+"
	 ;; Match signal names
	 "\\([a-zA-Z][a-zA-Z0-9_.,() \t+-]*\\)"
	 ;; Skip over signal direction, if any
	 ":[ \t]*\\(inout \\|out \\|in \\|buffer \\)?[ \t]*"
	 ;; Match signal type
	 "[a-zA-Z][a-zA-Z0-9_.'() \t+-]+"
	 ;; Find the statement terminator
	 "[:;\n]")
	2 'font-lock-variable-name-face)


       ;; Fontify names of ports in entity declarations The first case
       ;; assumes no signal statement and relies on finding a port
       ;; direction.
       ;;
       ;; Note that this expression will mistakenly match some
       ;; constant, variable and file declarations. We rely on the
       ;; fact that these have already been highlighted by the
       ;; expression above and will not be highlighted again.
       (list
	(concat
	 ;; Skip over any declarations, including signal statement
	 "^[a-zA-Z0-9_ \t]*[ \t(]+\\(signal \\)?[ \t]*"
	 ;; Match signal names
	 "\\([a-zA-Z][a-zA-Z0-9_.,() \t+-]*\\)"
	 ;; Skip over signal direction
	 ":[ \t]*\\(inout \\|out \\|in \\|buffer \\)[ \t]*"
	 ;; Match signal type
	 "[a-zA-Z][a-zA-Z0-9_.'() \t+-]+"
	 ;; Find the statement terminator
	 "[:;\n]")
	2 'font-lock-signal-name-face)


       ;; Fontify signal names in stand-alone signal declarations,
       ;; i.e. outside of entity, function & procedure
       ;; declarations. This expression ignores port direction and
       ;; relies on finding the signal statement. May overlap with the
       ;; expression for entity declrations above, but no matter.
       (list
	(concat
	 ;; Skip over signal statment.
	 "^[ \t]*signal[ \t]+"
	 ;; Match the signal names and check for terminator.
	 "\\([a-zA-Z][a-zA-Z0-9_.,() \t+-]*\\):")
	1 'font-lock-signal-name-face)


       ;; Fontify types of ports in entity declarations The first case
       ;; assumes no signal statement and relies on finding a port
       ;; direction.
       (list
	(concat
	 ;; Skip over any declarations, signal names & port direction
	 "^[a-zA-Z0-9_.,() \t]+:[ \t]*"
	 "\\(inout \\|out \\|in \\|buffer \\)[ \t]*"
	 ;; Match signal type
	 "\\([a-zA-Z][a-zA-Z0-9_.'() \t+-]+\\)"
	 ;; Find the statement terminator
	 "[:;\n]")
	2 'font-lock-type-face)



       ;; This case ignores port direction, and looks for signal
       ;; declarations. Thus it also covers non-port signal declaration.
       (list
	(concat
	 ;; Skip over optional port declaration and find signal decl.
	 "^[a-zA-Z0-9_.,() \t]*[( \t]+signal[ \t]+"
	 ;; Skip over the signal names, terminator and optional direction
	 "[a-zA-Z][a-zA-Z0-9_.,() \t+-]*:[ \t]*"
	 "\\(inout \\|out \\|in \\|buffer \\)?[ \t]*"
	 ;; Match signal type
	 "\\([a-zA-Z][a-zA-Z0-9_.'() \t+-]+\\)"
	 ;; Find the statement terminator
	 "[:;\n]")
	2 'font-lock-type-face)


       ;; Fontify signal names where they are used.
       (list
	(concat
	 ;; Look for the signal name first, and highlight. Name in
	 ;; this case may include parentheses
	 "^[ \t]*\\([a-zA-Z][a-zA-Z0-9_.() \t+-]*\\)"
	 ;; Lastly, check for the signal assignment
	 "[ \t]*<=")
	1 'font-lock-signal-name-face)


       ;; Fontify names of types and subtypes being defined.
       (list
	(concat
	 ;; Look for the declaration first
	 "^[ \t]*\\(type\\|subtype\\|end[ \t]+record\\)[ \t]+"
	 ;; Now, look for the type label & highlight
	 "\\([a-zA-Z][a-zA-Z0-9_]*\\)")
	2 'font-lock-type-face)


       ;; Fontify values of types and subtypes being defined.
       (list
	(concat
	 ;; Look for the declaration first
	 "^[ \t]*\\(type\\|subtype\\)[ \t]+"
	 ;; Skip over the type label
	 "[a-zA-Z][a-zA-Z0-9_]*[ \t]+is[ \t]+"
	 ;; Highlight the value
	 "\\([^:;\n]+\\)"
	 ;; Look for the terminator
	 ";")
	2 'font-lock-type-face)


       ;; Fontify types and subtypes in variable & constant declarations
       (list
	(concat
	 ;; Look for the declaration first. It may be inside
	 ;; parentheses if it is part of an entity declaration.
	 "^[ \t]*\\(variable\\|constant\\)[ \t]+"
	 ;; Skip over the signal names & terminator
	 "[a-zA-Z][a-zA-Z0-9_.,() \t+-]*:[ \t]*"
	 ;; Match variable/constant type
	 "\\([a-zA-Z][a-zA-Z0-9_.'() \t+-]+\\)"
	 ;; Find the statement terminator
	 "[ \t]*[:;]")
	2 'font-lock-type-face)


       ;; Fontify function return types being declared
       (list
	(concat
	 ;; Look for the declaration first
	 "^.*[ \t]+return[ \t]+"
	 ;; Now, look for the type label & highlight
	 "\\([a-zA-Z][a-zA-Z0-9_]*\\)"
	 ;; Now check for the declaration terminator
	 "\\([ \t]*;\\|[ \t]+is\\>[^\\s_]\\)")
	1 'font-lock-type-face)


       ;; Fontify variables being returned from functions
       (list
	(concat
	 ;; Look for the declaration first
	 "^[ \t]*return[ \t]+"
	 ;; Now, look for the type label & highlight
	 "\\([^;\n]+\\)"
	 ;; Now check for the declaration terminator
	 ";")
	1 'font-lock-type-face)
       ))


;; Finally some level-3 fontifying definitions too.
(setq vhdl-font-lock-keywords-3
      (list


       ;; Fontify reserved words.
       (list
	(concat
	 "\\(^\\|[^\\s_]\\)\\<\\("
	 (mapconcat
	  'identity
	  '(
	    ;; the following is a list of all reserved words known in VHDL'93
	    "abs" "access" "after" "alias" "all" "and" "assert"
	    "architecture" "array" "attribute"
	    "begin" "block" "body" "buffer" "bus"
	    "case" "component" "configuration" "constant"
	    "disconnect" "downto"
	    "else" "elsif" "end" "entity" "exit"
	    "file" "for" "function"
	    "generate" "generic" "group" "guarded"
	    "if" "impure" "in" "inertial" "inout" "is"
	    "label" "library" "linkage" "literal" "loop"
	    "map" "mod"
	    "nand" "new" "next" "nor" "not" "null"
	    "of" "on" "open" "or" "others" "out"
	    "package" "port" "postponed" "procedure" "process" "pure"
	    "range" "record" "register" "reject" "rem" "report" "resolved"
	    "return" "rol" "ror"
	    "select" "severity" "signal" "shared" "sla" "sll" "sra" "srl"
	    "subtype"
	    "then" "to" "transport" "type"
	    "unaffected" "units" "until" "use"
	    "variable" "wait" "when" "while" "with"
	    "xnor" "xor"
	    ;; the following list contains predefined attributes
	    "base" "left" "right" "high" "low" "pos" "val" "succ"
	    "pred" "leftof" "rightof" "range" "reverse_range"
	    "length" "delayed" "stable" "quiet" "transaction"
	    "event" "active" "last_event" "last_active" "last_value"
	    "driving" "driving_value" "ascending" "value" "image"
	    "simple_name" "instance_name" "path_name"
	    "foreign"
	    ;; the following list contains standardized types
	    "boolean" "bit" "bit_vector" "character" "severity_level" "integer"
	    "real" "time" "natural" "positive" "string" "text" "line"
	    "unsigned" "signed"
	    "std_logic" "std_logic_vector"
	    "std_ulogic" "std_ulogic_vector"
	    ;; the following list contains standardized enumeration
	    ;; values
	    "true" "false"
	    "note" "warning" "error" "failure"
	    "fs" "ps" "ns" "us" "ms" "sec" "min" "hr"
	    ;; TEXTIO functions
	    "readline" "writeline" "read" "write" "endline" "endfile"
	    )
	  "\\|")
	 "\\)\\>[^\\s_]")
	2 'font-lock-keyword-face)
       ))


;; Default to the medium CPU-intensive list of keywords.
(defvar vhdl-font-lock-keywords (append vhdl-font-lock-keywords-1
					vhdl-font-lock-keywords-2)
  "Expressions to highlight in VHDL mode.")

(defun vhdl-font-lock-update-decoration-level ()
  "Set the VHDL font-lock patterns to that requested by the user."
  (cond ((eq vhdl-font-lock-highlight-level nil)
	 (setq vhdl-font-lock-keywords nil))
	((eq vhdl-font-lock-highlight-level 1)
	 (setq vhdl-font-lock-keywords vhdl-font-lock-keywords-1))
	((eq vhdl-font-lock-highlight-level 2)
	 (setq vhdl-font-lock-keywords (append vhdl-font-lock-keywords-1
					       vhdl-font-lock-keywords-2)))
	((eq vhdl-font-lock-highlight-level 3)
	 (setq vhdl-font-lock-keywords (append vhdl-font-lock-keywords-1
					       vhdl-font-lock-keywords-2
					       vhdl-font-lock-keywords-3)))))


;; Some ugly stuff that is necessary to support GNU Emacs as well as
;; XEmacs.

(defun vhdl-highlight-GNU-emacs-additions ()
  (make-local-variable 'font-lock-keywords-case-fold-search)
  (setq font-lock-keywords-case-fold-search	t)
  (setq font-lock-keywords			vhdl-font-lock-keywords)
  (font-lock-mode 1))


;; Set the flag once the file has been loaded successfully.
(provide 'vhdl-highlight)
